@@ -18,20 +18,20 @@
 '''
 
 __title__ = 'surveygizmo'
-__version_info__ = ('0', '1', '2')
+__version_info__ = ('0', '2', '0')
 __version__ = '.'.join(__version_info__)
 __author__ = 'Ryan P Kilby'
-__license__ = 'MIT'
-__copyright__ = 'Copyright 2013 Ryan P Kilby'
+__license__ = 'BSD 3-Clause'
+__copyright__ = 'Copyright 2013-2014 NC State University'
 __all__ = ['SurveyGizmo', 'ImproperlyConfigured', 'SGAuthService']
 
 
-# Hack from rauth. This is necessary for imports during setup/install
+# Import pattern from rauth. This is necessary for imports during setup/install
 try:
     from .oauth_helper import SGAuthService
-    from .surveygizmo import SurveyGizmo, ImproperlyConfigured
+    from .surveygizmo import SurveyGizmo, ImproperlyConfigured, default_52x_handler
 
     # placate pyflakes
-    (SGAuthService, SurveyGizmo, ImproperlyConfigured)
+    (SGAuthService, SurveyGizmo, ImproperlyConfigured, default_52x_handler)
 except ImportError:  # pragma: no cover
     pass
