@@ -1,8 +1,13 @@
 
+import collections
+import hashlib
+import logging
+import time
+
 import oauth_helper
 import requests
-import hashlib
-import imp
+
+logger = logging.getLogger(__name__)
 
 
 class ImproperlyConfigured(Exception):
@@ -10,15 +15,29 @@ class ImproperlyConfigured(Exception):
     pass
 
 
-class InvalidFilter(Exception):
-    """ Filter format is invalid."""
-    pass
+def default_52xhandler(response, requests_kwargs, response_type):
+    """ Default 52x handler that loops every second until a non 52x response is received.
+        :param response: The response of the last executed request
+        :param response_type: The expected response format. Defaults to config.response_type.
+    """
+    time.sleep(1)
+    response = requests.get(response.url, **requests_kwargs)
+
+    if 520 <= response.status_code < 530:
+        return default_52xhandler(response, requests_kwargs, response_type)
+
+    response.raise_for_status()
+
+    if not response_type:
+        return response.json()
+    else:
+        return response.text()
 
 
-class _Config(object):
+class Config(object):
     def __init__(self, _sg, **kwargs):
         self._sg = _sg
-        self._api_version = kwargs.get('api_version', 'head')
+        self.api_version = kwargs.get('api_version', 'head')
         self.auth_method = kwargs.get('auth_method', None)
         self.username = kwargs.get('username', None)
         self.password = kwargs.get('password', None)
@@ -27,14 +46,16 @@ class _Config(object):
         self.consumer_secret = kwargs.get('consumer_secret', None)
         self.access_token = kwargs.get('access_token', None)
         self.access_token_secret = kwargs.get('access_token_secret', None)
+
         self.response_type = kwargs.get('response_type', None)
+        self.requests_kwargs = kwargs.get('requests_kwargs', {})
+        self.prepare_url = kwargs.get('prepare_url', False)
+        self.preserve_filters = kwargs.get('preserve_filters', False)
+        self.handler52x = kwargs.get('handler52x', None)
 
     def validate(self):
         """ Perform validation check on properties.
         """
-        if not self.api_version in ['v3', 'head']:
-            raise ImproperlyConfigured("API version provided is invalid.")
-
         if not self.auth_method in ['user:pass', 'user:md5', 'oauth']:
             raise ImproperlyConfigured("No authentication method provided.")
         else:
@@ -54,108 +75,41 @@ class _Config(object):
         if not self.response_type in ["json", "pson", "xml", "debug", None]:
             raise ImproperlyConfigured()
 
-    @property
-    def api_version(self):
-        return self._api_version
 
-    @api_version.setter
-    def api_version(self, value):
-        self._api_version = value
-        self._sg.api._check_version()
-
-
-class _Field(object):
-    associations = {
-        'surveyresponse': [
-            '[question(%d)]',
-            '[question(%d), option(%d)]'
-            'datesubmitted',
-            'istestdata',
-            'status',
-            'contact_id',
-        ],
-        'survey': [
-            'createdon',
-            'modifiedon',
-            'title',
-            'subtype',
-            'team',
-            'status',
-        ],
-        'surveycampaign': [
-            '_type',
-            'name',
-            'ssl',
-            'datecreated',
-            'datemodified',
-            'status',
-        ],
-    }
-
-
-class _API(object):
+class API(object):
     base_url = "https://restapi.surveygizmo.com/"
-    head = 'v3'
 
-    def __init__(self, _sg):
-        self._sg = _sg
-        self._api_version = None
-        self._modules = None
+    def __init__(self, config):
+        self._config = config
+
+        self._resources = {}
         self._filters = []
         self._session = None
 
-    def _check_version(self):
-        """ update api modules, wrap callables.
+        self._import_api()
+
+    def _import_api(self):
+        """ update api resources, wrap callables.
         """
-        if self._api_version != self._sg.config.api_version:
-            self._api_version = self._sg.config.api_version
-            self._modules = {}
+        resources = __import__('api', globals(), locals(), ['*'])
 
-            api_version = self.head if self._api_version == 'head' else self._api_version
+        for resource_name in resources.__all__:
+            resource = getattr(resources, resource_name)
 
-            module_list = getattr(__import__('api', globals(), locals(), [api_version]), api_version).__all__
-            _, package_path, _ = imp.find_module('surveygizmo')
-            search_path = '%s/api/%s' % (package_path, api_version)
+            for name, func in resource.__dict__.items():
+                if isinstance(func, collections.Callable) and not name.startswith('__'):
+                    setattr(resource, name, self._wrap(func))
 
-            for module_name in module_list:
-                find_result = imp.find_module(module_name, [search_path])
-                module = imp.load_module("_api_loaded_%s" % module_name, *find_result)
-
-                for name, func in module.__dict__.items():
-                    if callable(func) and not name.startswith('__'):
-                        setattr(module, name, self._wrap(func))
-
-                self._modules[module_name] = module
+            self._resources[resource_name] = resource
 
     def __getattr__(self, name):
-        """ retrieve modules loaded from api
+        """ retrieve resources loaded from api
         """
-        if self._modules.get(name, None) is not None:
-            return self._modules[name]
+        if self._resources.get(name, None) is not None:
+            return self._resources[name]
         raise AttributeError(name)
 
-    def _wrap(self, func):
-        """ wrap api callable's such that their return values
-            are immediately executed
-        """
-        def wrapper(*args, **kwargs):
-            keep = kwargs.pop('keep', False)
-            url_fetch = kwargs.pop('url_fetch', False)
-
-            tail, params = func(*args, **kwargs)
-
-            response_type = self._sg.config.response_type
-            if response_type:
-                tail = "%s.%s" % (tail, response_type)
-            tail = "%s/%s" % (self._api_version, tail)
-
-            vals = self.prepare(tail, params, keep)
-            if url_fetch:
-                return vals
-            return self.execute(*vals)
-        return wrapper
-
-    def add_filter(self, field, operator, value):  # , object_type=None):
+    def add_filter(self, field, operator, value):
         """ Add a query filter to be applied to the next API call.
             :param field: Field name to filter by
             :type field: str
@@ -207,21 +161,55 @@ class _API(object):
             'filter[value][%d]' % i: str(value),
         })
 
-    def prepare(self, tail, params, keep=False):
+    def get_filters(self):
+        params = {}
+        for _filter in self._filters:
+            params.update(_filter)
+        if not self._config.preserve_filters:
+            self.clear_filters()
+
+        return params
+
+    def clear_filters(self):
+        self._filters = []
+
+    def _wrap(self, func):
+        """ wrap api callable's such that their return values
+            are immediately executed
+        """
+        # in case of repeat imports, prevents re-wrapping object
+        if hasattr(func, '_wrapped') and func._wrapped:
+            logger.debug('attempting to rewrap %s' % str(func))
+            return func
+
+        def wrapped(*args, **kwargs):
+            tail, params = func(*args, **kwargs)
+
+            if self._config.response_type:
+                tail = "%s.%s" % (tail, self._config.response_type)
+            tail = "%s/%s" % (self._config.api_version, tail)
+
+            vals = self.prepare(tail, params)
+            if self._config.prepare_url:
+                return vals
+            return self.execute(*vals)
+
+        wrapped.__name__ = func.__name__
+        wrapped.__doc__ = func.__doc__
+        wrapped._wrapped = True
+        return wrapped
+
+    def prepare(self, tail, params):
         """ Prepares the url and remaining params for execution
             :param tail: The tail portion of the URL. This should not include
             the domain name.
             :param params: Query parameters passed to API.
-            :param keep: Keep filters for next API call. Defaults to False.
         """
-        self._sg.config.validate()
+        config = self._config
+        config.validate()
 
-        for _filter in self._filters:
-            params.update(_filter)
-        if not keep:
-            self._filters = []
+        params.update(self.get_filters())
 
-        config = self._sg.config
         if config.auth_method == 'user:pass':
             params.update({
                 config.auth_method: "%s:%s" % (config.username, config.password),
@@ -242,7 +230,7 @@ class _API(object):
             :param url: The full url for the api call.
             :param params: Query parameters passed to API.
         """
-        config = self._sg.config
+        config = self._config
         if config.auth_method == 'oauth':
             if not self._session:
                 self._session = oauth_helper.SGAuthService(
@@ -250,9 +238,13 @@ class _API(object):
                     config.access_token, config.access_token_secret
                 ).get_session()
 
-            response = self._session.get(url, params)
+            response = self._session.get(url, params=params, **config.requests_kwargs)
         else:
-            response = requests.get(url, params=params)
+            response = requests.get(url, params=params, **config.requests_kwargs)
+
+        if 520 <= response.status_code < 530:
+            if self._config.handler52x:
+                return self._config.handler52x(response, config.requests_kwargs, config.response_type)
 
         response.raise_for_status()
 
@@ -263,9 +255,8 @@ class _API(object):
 
 
 class SurveyGizmo(object):
-    """
+    """ SurveyGizmo API client.
     """
     def __init__(self, **kwargs):
-        self.config = _Config(self, **kwargs)
-        self.api = _API(self)
-        self.api._check_version()
+        self.config = Config(self, **kwargs)
+        self.api = API(self.config)
